/* A C program used to .coe (Single Port RAM) file generated from KCPSMBLE */
/* into .coe (Dual Port RAM) file and .TBP file */ 
/* There is an optional verbose mode [-v] */
/* By Henry Fu */
 
#include <stdio.h> 
#include <stdlib.h> 
#include <errno.h> 
#include <string.h> 
#include <sys/types.h> 

/* Globals for getting command line arguments */
static char *options = "v";
static char *usage = "[input filename] [-v]";
int vflg = 0, errflg = 0;

/* Required by getopt */
extern char *optarg;
extern int optind, opterr;

/* Globals */
char *lbuf, *filename;
int *vtr;

/* Function to get command line arguments */
void parseargs(int argc, char *argv[])
{
    int c;
    int ok = 1;

    filename = argv[optind++];

    while ((c=getopt(argc,argv,options)) != -1)
        switch (c)
        {
        case 'v':
          vflg++;
          break;
        default:
          ok = 0;
        }
   
    if (!ok || optind < argc) {
      fprintf (stderr, "usage : %s %s\n", argv[0], usage);
      exit(0);
    }
}

/* Function to read input */
void parseinput()
{
    int i = 0;
    FILE *fd;

    /* open file */
    if (filename != NULL) {
      if ((fd = fopen(filename,"r")) == NULL) {
        fprintf (stderr, "Cannot open file %s.\n", filename);
        exit(1);
      }
    }

    /* Parse Header */
    for (i=0; i<6; i++)
      fgets(lbuf, 80, fd);

    /* Get Memory Initialization Vector */
    for (i=0; i<256; i+=16) {
      fscanf(fd, "%4x, %x, %x, %x, %x, %x, %x, %x, ", 
            &vtr[i], &vtr[i+1], &vtr[i+2], &vtr[i+3],
            &vtr[i+4], &vtr[i+5], &vtr[i+6], &vtr[i+7]);
      fscanf(fd, "%x, %x, %x, %x, %x, %x, %x, %x, \n", 
            &vtr[i+8], &vtr[i+9], &vtr[i+10], &vtr[i+11],
            &vtr[i+12], &vtr[i+13], &vtr[i+14], &vtr[i+15]);
    }

    fclose(fd);
}

/* Function to write .COE file */
void writecoe()
{
    int i = 0;
    FILE *fp = fopen("program.coe", "w");

    /* Write Component Name */
    fprintf (fp, "Component_Name=%s;\n", "program");

    /* Write Data Width A */
    fprintf (fp, "Data_width_A=%d;\n", 16);

    /* Write Depth A */
    fprintf (fp, "Depth_A=%d;\n", 512);

    /* Write Data Width B */
    fprintf (fp, "Data_width_B=%d;\n", 32);

    /* Write Depth B */
    fprintf (fp, "Depth_B=%d;\n", 256);

    /* Write Radix */
    fprintf (fp, "Radix=%d;\n", 16);

    /* Write Default Data */
    fprintf (fp, "Default_Data=%.4d;\n", 0);

    /* Write Memory Initialization Vector */
    fprintf (fp, "Memory_Initialization_Vector=\n");
    for (i=0; i<240; i+=16) {
      fprintf (fp, "%.4X, %.4X, %.4X, %.4X, %.4X, %.4X, %.4X, %.4X, ",
                   vtr[i], vtr[i+1], vtr[i+2], vtr[i+3],
                   vtr[i+4], vtr[i+5], vtr[i+6], vtr[i+7]);
      fprintf (fp, "%.4X, %.4X, %.4X, %.4X, %.4X, %.4X, %.4X, %.4X, \n",
                   vtr[i+8], vtr[i+9], vtr[i+10], vtr[i+11],
                   vtr[i+12], vtr[i+13], vtr[i+14], vtr[i+15]);
    }
    fprintf (fp, "%.4X, %.4X, %.4X, %.4X, %.4X, %.4X, %.4X, %.4X, ",
                 vtr[i], vtr[i+1], vtr[i+2], vtr[i+3],
                 vtr[i+4], vtr[i+5], vtr[i+6], vtr[i+7]);
    fprintf (fp, "%.4X, %.4X, %.4X, %.4X, %.4X, %.4X, %.4X, %.4X;",
                 vtr[i+8], vtr[i+9], vtr[i+10], vtr[i+11],
                 vtr[i+12], vtr[i+13], vtr[i+14], vtr[i+15]);

    fclose (fp);
}

void writeTBP()
{
    int i=0;
    FILE *fp = fopen ("INST.TBP", "w");

    /* Write Comment */
    fprintf (fp, "# demo testbench\n");
    fprintf (fp, "#\n");
    fprintf (fp, "# comments start with #\n");
    fprintf (fp, "# commands start with !\n");
    fprintf (fp, "# parameters are optional\n");
    fprintf (fp, "#\n");
    fprintf (fp, "# UDP block, will be prepended by an UDP header\n");
    fprintf (fp, "# parameter: ip-address, dest-port, src-port\n");
    fprintf (fp, "!UDP 192.168.10.1 7 64000\n");
    fprintf (fp, "00000000\n");

    /* Write Program Vector */
    for (i=0; i<256; i+=2) {
      /* if ((vtr[i] != 0) || (vtr[i+1] != 0)) */
         fprintf (fp, "%.4X%.4X\n",vtr[i+1],vtr[i]);
    } 

    fclose (fp);
}

int main(int argc, char *argv[])
{
    lbuf = calloc(80, sizeof(char));
    vtr = (int *) calloc(256, sizeof(int));

    parseargs(argc, argv);
    parseinput();
    writecoe();
    writeTBP();

    return 0;
}
