-- file: fake_NID_in.vhd
-- simulation tool to capture cells released to NID
-- Michael Wrighton, mgw1@arl.wustl.edu
-- Washington University
-- Applied Research Lab
-- Created: 3/17/2000

-------------------------------------------------------------------------------
-- Modified 6/20/2001 by det3
--
-- Added generic for specifying output filename.
-------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;
USE ieee.std_logic_textio.ALL;
USE std.textio.ALL;

ENTITY fake_NID_in IS
  generic (outfile : string := "CELLSOUT.DAT");  -- output file
  PORT (
    clock, soc_in : IN  STD_LOGIC;
    tcaff         : OUT STD_LOGIC;
    data_in       : IN  STD_LOGIC_VECTOR(31 DOWNTO 0));

END fake_NID_in;


ARCHITECTURE behavioral OF fake_NID_in IS

  file out_file : text open write_mode is outfile;

  signal word_count : integer := 13;

BEGIN  -- behavioral
  tcaff <= '1';                         -- always accept data
  write_process : process(clock)
    variable line_out : LINE;
    VARIABLE newcell  : STRING(8 DOWNTO 1) := "new_cell";
    
  begin
    if (clock = '1' and clock'event) then
      if soc_in = '1' then
        assert word_count = 13 report "Multiple SOCs in 1 Cell Time" severity WARNING;
        word_count <= 0;
        write(line_out, newcell);
        writeline(out_file, line_out);
      end if;
      if soc_in = '1' or word_count < 13 then
        hwrite(line_out, data_in);      -- put the hex value in a line buffer
        writeline(out_file, line_out);
        if soc_in = '0' then
          word_count <= word_count + 1;
        end if;
      end if;
    end if;
  end process;
END behavioral;
