-- file: fake_NID_out.vhd
-- simulation tool to generate cells transmitted by NID
-- Michael Wrighton, mgw1@arl.wustl.edu
-- Washington University
-- Applied Research Lab
-- Created: 3/17/2000

-------------------------------------------------------------------------------
-- Modified 6/20/2001 by det3
--
-- Added generic for specifying input filename.
-------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;
USE ieee.std_logic_textio.ALL;
USE std.textio.ALL;

ENTITY fake_NID_out IS
  generic (
    infile : string := "TESTCELL.DAT");  -- input file
  PORT (
    clock    : IN  STD_LOGIC;
    reset_l  : IN  STD_LOGIC;
    data_out : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    soc_out  : OUT STD_LOGIC;
    tcaff    : in  std_logic);

END fake_NID_out;


ARCHITECTURE behavioral OF fake_NID_out IS

  CONSTANT period : time := 50 ns;
  file in_file    : text open read_mode is infile;

  signal word_count : integer := 0;

BEGIN  -- behavioral
  soc_out <= 'L';
--   clkprc: PROCESS
--     VARIABLE c : STD_LOGIC := '0';
--   BEGIN
--     WHILE true LOOP
--       WAIT FOR (period / 2);
--         c := NOT c;
--         clock <= c;
--     END LOOP;
--   END PROCESS clkprc;


  read_process : PROCESS
    
    VARIABLE line_in : LINE;
    VARIABLE a       : STD_LOGIC_VECTOR(31 DOWNTO 0);
    VARIABLE s       : STRING(8 DOWNTO 1) := "xxxxxxxx";
    
  BEGIN
    WAIT UNTIL (clock'event AND clock = '1');
    IF (reset_l = '1' and not endfile(in_file)) THEN
      soc_out <= '0';
      readline(in_file, line_in);       -- read a line from file
      assert not endfile(in_file) report "Last Line Read" severity NOTE;
      read(line_in, s);
      IF (s = "new_cell") THEN
        while (tcaff = '0') loop
          WAIT UNTIL (clock'event AND clock = '1');
        end loop;
        readline(in_file, line_in);
        hread(line_in, a);
        soc_out <= '1';
        s       := "xxxxxxxx";
      ELSIF (s = "wait1000") THEN
        for i in 0 to 1000 loop
          WAIT UNTIL (clock'event AND clock = '1');
        end loop;
      ELSIF (s = "wait_100") THEN
        for i in 0 to 100 loop
          WAIT UNTIL (clock'event AND clock = '1');
        end loop;
      ELSIF (s = "wait__10") THEN
        for i in 0 to 100 loop
          WAIT UNTIL (clock'event AND clock = '1');
        end loop;
      ELSE
        write(line_in, s);              -- put back what we read into line
        hread(line_in, a);
        soc_out <= '0';
      END IF;
      
      data_out <= a;
    ELSE
      soc_out <= '0';
    END IF;
    
  END PROCESS read_process;
  
END behavioral;
