-- testbench.vhd
-- top-level RAD simulation
-- components:
--   rad.vhd, fake_NID_*, SRAM model, clock


LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

ENTITY testbench IS
  -- nothing
END testbench;

ARCHITECTURE simulation OF testbench IS
  component rad_loopback
    port (
      RAD_CLK      : IN    STD_LOGIC;
      RAD_CLKB     : IN    STD_LOGIC;
      RAD_RESET    : IN    STD_LOGIC;
      RAD_READY    : OUT   STD_LOGIC;
      RAD_RECONFIG : INOUT STD_LOGIC_VECTOR(2 DOWNTO 0);
      SOC_LC_NID   : IN    STD_LOGIC;
      D_LC_NID     : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
      TCAFF_LC_RAD : OUT   STD_LOGIC;
      SOC_LC_RAD   : OUT   STD_LOGIC;
      D_LC_RAD     : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0);
      TCAFF_LC_NID : IN    STD_LOGIC;
      SOC_SW_NID   : IN    STD_LOGIC;
      D_SW_NID     : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
      TCAFF_SW_RAD : OUT   STD_LOGIC;
      SOC_SW_RAD   : OUT   STD_LOGIC;
      D_SW_RAD     : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0);
      TCAFF_SW_NID : IN    STD_LOGIC;
      RAD_TEST1    : INOUT STD_LOGIC_VECTOR(15 DOWNTO 0);
      RAD_TEST2    : INOUT STD_LOGIC_VECTOR(15 DOWNTO 0));
  end component;

  COMPONENT clock
    PORT (
      clk : OUT STD_LOGIC);
  END COMPONENT;

  COMPONENT fake_NID_in
    generic (outfile : string := "CELLSOUT.DAT");  -- output file
    PORT (
      clock, soc_in : IN  STD_LOGIC;
      tcaff         : OUT STD_LOGIC;
      data_in       : IN  STD_LOGIC_VECTOR(31 DOWNTO 0));
  END COMPONENT;

  COMPONENT fake_NID_out
    generic (
      infile : string := "TESTCELL.DAT");  -- input file
    PORT (
      clock    : IN  STD_LOGIC;
      reset_l  : IN  STD_LOGIC;
      data_out : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
      soc_out  : OUT STD_LOGIC;
      tcaff    : IN  STD_LOGIC);
  END COMPONENT;

  signal   RAD_CLK_i      : STD_LOGIC;
  signal   RAD_RESET_i    : STD_LOGIC;
  signal   RAD_READY_i    : STD_LOGIC;
  signal   RAD_RECONFIG_i : STD_LOGIC_VECTOR(2 DOWNTO 0);
  signal   SOC_LC_NID_i   : STD_LOGIC;
  signal   D_LC_NID_i     : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal   TCAFF_LC_RAD_i : STD_LOGIC;
  signal   SOC_LC_RAD_i   : STD_LOGIC;
  signal   D_LC_RAD_i     : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal   TCAFF_LC_NID_i : STD_LOGIC;
  signal   SOC_SW_NID_i   : STD_LOGIC;
  signal   D_SW_NID_i     : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal   TCAFF_SW_RAD_i : STD_LOGIC;
  signal   SOC_SW_RAD_i   : STD_LOGIC;
  signal   D_SW_RAD_i     : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal   TCAFF_SW_NID_i : STD_LOGIC;
  signal   RAD_TEST2      : STD_LOGIC_VECTOR(15 DOWNTO 0);
  signal   RAD_TEST1      : STD_LOGIC_VECTOR(15 DOWNTO 0);
--
  SIGNAL   gnd            : STD_LOGIC := '0';
  SIGNAL   vcc            : STD_LOGIC := '1';
--
  constant period         : time      := 10 ns;
--
BEGIN

  RAD_LOOPBACK1 : rad_loopback
    port map (
      RAD_CLK      => RAD_CLK_i,
      RAD_CLKB     => RAD_CLK_i,
      RAD_RESET    => RAD_RESET_i,
      RAD_READY    => RAD_READY_i,
      RAD_RECONFIG => RAD_RECONFIG_i,
      SOC_LC_NID   => SOC_LC_NID_i,
      D_LC_NID     => D_LC_NID_i,
      TCAFF_LC_RAD => TCAFF_LC_RAD_i,
      SOC_LC_RAD   => SOC_LC_RAD_i,
      D_LC_RAD     => D_LC_RAD_i,
      TCAFF_LC_NID => TCAFF_LC_NID_i,
      SOC_SW_NID   => SOC_SW_NID_i,
      D_SW_NID     => D_SW_NID_i,
      TCAFF_SW_RAD => TCAFF_SW_RAD_i,
      SOC_SW_RAD   => SOC_SW_RAD_i,
      D_SW_RAD     => D_SW_RAD_i,
      TCAFF_SW_NID => TCAFF_SW_NID_i,
      RAD_TEST2    => RAD_TEST2,
      RAD_TEST1    => RAD_TEST1);

  CLOCK1 : clock
    PORT MAP (
      clk => RAD_CLK_i);

  INGRESS_FAKE_NID_OUT : fake_NID_out
    generic map (
      infile => "INPUT_CELLS.DAT")
    PORT MAP (
      clock    => RAD_CLK_i,
      reset_l  => RAD_RESET_i,
      data_out => D_LC_NID_i,
      soc_out  => SOC_LC_NID_i,
      tcaff    => TCAFF_LC_RAD_i);

  INGRESS_FAKE_NID_IN : fake_NID_in
    generic map (
      outfile => "LC_CELLSOUT.DAT")
    PORT MAP (
      clock   => RAD_CLK_i,
      soc_in  => SOC_LC_RAD_i,
      tcaff   => TCAFF_LC_NID_i,
      data_in => D_LC_RAD_i);

  ENGRESS_FAKE_NID_OUT : fake_NID_out
    generic map (
      infile => "INPUT_CELLS.DAT")
    PORT MAP (
      clock    => RAD_CLK_i,
      reset_l  => RAD_RESET_i,
      data_out => D_SW_NID_i,
      soc_out  => SOC_SW_NID_i,
      tcaff    => TCAFF_SW_RAD_i);

  ENGRESS_FAKE_NID_IN : fake_NID_in
    generic map (
      outfile => "SW_CELLSOUT.DAT")
    PORT MAP (
      clock   => RAD_CLK_i,
      soc_in  => SOC_SW_RAD_i,
      tcaff   => TCAFF_SW_NID_i,
      data_in => D_SW_RAD_i);

  process
  begin
    WAIT FOR period/2;
    RAD_RESET_i       <= '0';
    RAD_RECONFIG_i(0) <= '1';
    RAD_RECONFIG_i(1) <= '0';
    WAIT FOR period;
    RAD_RESET_i       <= '1';
    wait for period*10;
    WAIT;
  end process;

END simulation;

-- configuration testbench_conf of testbench is
--  for simulation
--    for all : rad_loopback
--      use configuration work.rad_loopback_conf;
--    end for;
--  end for;
-- end testbench_conf;
