LIBRARY ieee;
  USE ieee.std_logic_1164.ALL;
  USE ieee.std_logic_arith.ALL;
  use IEEE.STD_LOGIC_UNSIGNED.all;

ENTITY blink IS
  PORT (
    clk1    : IN  STD_LOGIC;
    clk2    : IN  STD_LOGIC;
    reset_l : IN  STD_LOGIC;
    led1    : OUT STD_LOGIC;
    led2    : OUT STD_LOGIC
  );
END blink;

ARCHITECTURE behav OF blink IS

signal led1_27    : std_logic_vector(26 downto 0);
signal led2_27    : std_logic_vector(26 downto 0);

BEGIN

-- * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
-- LED1 is driven by a 27-bit counter clocked by the RAD system clock (100MHz)
-- LED2 is driven by a 27-bit counter clocked by the RAD system clock B (100MHz)
-- * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

-- a flip-flop is added after the last counter stage so that it can 
-- sit in the IOB.

-- LED1 logic
led1_proc : PROCESS (clk1, reset_l)
  BEGIN
    IF (reset_l = '0') THEN
      led1 <=  '1';
    ELSIF (clk1'event AND clk1 = '1') THEN
      led1 <= led1_27(26);
    END IF;
  END PROCESS;
  

  
-- divide by 67 million
led1_cntr_proc : PROCESS (clk1, reset_l)
  BEGIN
    IF (reset_l = '0') THEN
      led1_27 <= (others => '0');
    ELSIF (clk1'event AND clk1 = '1') THEN
      led1_27 <= unsigned (led1_27) + 1;
    END IF;
  END PROCESS;
  
-- * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
-- LED2 logic
led2_proc : PROCESS (clk2, reset_l)
  BEGIN
    IF (reset_l = '0') THEN
      led2 <=  '1';
    ELSIF (clk2'event AND clk2 = '1') THEN
      led2 <= led2_27(26);
    END IF;
  END PROCESS;
  

  
-- divide by 67 million
led2_cntr_proc : PROCESS (clk2, reset_l)
  BEGIN
    IF (reset_l = '0') THEN
      led2_27 <= (others => '0');
    ELSIF (clk2'event AND clk2 = '1') THEN
      led2_27 <= unsigned (led2_27) + 1;
    END IF;
  END PROCESS;

END behav;
