--  Applied Research Laboratory
--  Washington University in St. Louis
--
--  File: loopback_module.vhd
--  Entity declaration for RAD module
--  Created by:  David E. Taylor (det3@arl.wustl.edu)
--  Created on:  August, 16 2000
--  Last modified:  August 18, 2000 @ 11:20 am
--
--  IMPORTANT:  Refer to RAD Module Interface Specification
--  for explanations and timing specifications for all interface
--  signals.
--
library IEEE;
use IEEE.STD_LOGIC_1164.all;
--
ENTITY loopback_module IS
  PORT (
    -- Clock & Reset
    clk     : in STD_LOGIC;             -- 100MHz global clock
    reset_l : in STD_LOGIC;             -- Synchronous reset, asserted-low

    -- Cell Input Interface
    soc_mod_in   : in  STD_LOGIC;                      -- Start of cell
    d_mod_in     : in  STD_LOGIC_VECTOR(31 downto 0);  -- 32-bit data
    tca_mod_in   : out STD_LOGIC;                      -- Transmit cell available

    -- Cell Output Interface
    soc_out_mod   : out STD_LOGIC;                      -- Start of cell
    d_out_mod     : out STD_LOGIC_VECTOR(31 downto 0);  -- 32-bit data
    tca_out_mod   : in  STD_LOGIC;                       -- Transmit cell available
    test_data     : out STD_LOGIC_VECTOR(31 downto 0)   -- 32-bit data
    );
end loopback_module;

architecture behavioral of loopback_module is

  signal soc : std_logic;               -- Start of Cell
  signal data : std_logic_vector(31 downto 0);  -- 32-bit words of ATM cell
  signal tca : std_logic;               -- Transmit Cell Available
--
begin  -- behavioral
--
-- Pass cells through.  On reconfig, hold TCA low.
--
  CELL_IO_FF : process (clk)
  begin  -- process CELL_FF
    if (clk='1' and clk'event) then
      if reset_l='0' then
        soc <= '0';
        soc_out_mod <= '0';
        data <= (others=>'0') ;
        d_out_mod <= (others=>'0') ;
        tca <= '0' ;
        tca_mod_in <= '0' ;
      else
        soc <= soc_mod_in ;
        data <= d_mod_in ;
        soc_out_mod <= soc ;
        d_out_mod <= data ;
        tca <= tca_out_mod ;
        tca_mod_in <= tca ;
      end if;
    end if;
  end process CELL_IO_FF;
  --
  test_data <= data;
--
end behavioral;
