--  Applied Research Laboratory
--  Washington University in St. Louis
--
--  File:  rad_loopback.vhd
--  Top level structure for RAD FPGA with ingress/egress loopback modules
--  Created by:  John W. Lockwood (lockwood@arl.wustl.edu),
--  David E. Taylor (det3@arl.wustl.edu)
--
-------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
-- synthesis translate_off
LIBRARY unisim;
-- synthesis translate_on
-------------------------------------------------------------------------------
ENTITY rad_loopback IS
  PORT (
    -- Clocks
    RAD_CLK  : IN STD_LOGIC;
    RAD_CLKB : IN STD_LOGIC;

    -- Reset & Reconfig
    RAD_RESET : IN  STD_LOGIC;
    RAD_READY : OUT STD_LOGIC;
    RAD_RECONFIG : INOUT STD_LOGIC_VECTOR(2 DOWNTO 0);

    -- NID Interface
    --
    -- Ingress Path
    -- Input
    SOC_LC_NID   : IN  STD_LOGIC;
    D_LC_NID     : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    TCAFF_LC_RAD : OUT STD_LOGIC;
    -- Output
    SOC_LC_RAD   : OUT STD_LOGIC;
    D_LC_RAD     : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    TCAFF_LC_NID : IN  STD_LOGIC;

    -- Egress Path
    -- Input
    SOC_SW_NID   : IN  STD_LOGIC;
    D_SW_NID     : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    TCAFF_SW_RAD : OUT STD_LOGIC;
    -- Output
    SOC_SW_RAD   : OUT STD_LOGIC;
    D_SW_RAD     : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    TCAFF_SW_NID : IN  STD_LOGIC;

    -- SRAM Module 1 (Ingress Side)
--    SRAM1_ADDR  : OUT   STD_LOGIC_VECTOR(17 DOWNTO 0);
--    SRAM1_DATA  : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
--    SRAM1_CE    : OUT   STD_LOGIC;        -- Chip Enable: LOW=drive
--    SRAM1_GW    : OUT   STD_LOGIC;        -- Global Write: LOW=write
--    SRAM1_PIN84 : OUT   STD_LOGIC;

    -- SRAM Module 2 (Egress side)
--    SRAM2_ADDR  : OUT   STD_LOGIC_VECTOR(17 DOWNTO 0);
--    SRAM2_DATA  : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
--    SRAM2_CE    : OUT   STD_LOGIC;        -- Chip Enable: LOW=drive
--    SRAM2_GW    : OUT   STD_LOGIC;        -- Global Write: LOW=write
--    SRAM2_PIN84 : OUT   STD_LOGIC;

    -- SDRAM Module 1 (Ingress side)
--    SDR1_ADDR   : OUT   STD_LOGIC_VECTOR(11 DOWNTO 0);
--    SDR1_DATA   : INOUT STD_LOGIC_VECTOR(63 DOWNTO 0);
--    SDR1_RAS    : OUT   STD_LOGIC;        -- Row Address Strobe
--    SDR1_CAS    : OUT   STD_LOGIC;        -- Column Address Strobe
--    SDR1_CE     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- Chip Enable
--    SDR1_BA     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  --
--    SDR1_S      : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- 
--    SDR1_SDA    : OUT   STD_LOGIC;        --
--    SDR1_SDC    : OUT   STD_LOGIC;        --
--    SDR1_WE     : OUT   STD_LOGIC;        -- Write Enable

    -- SDRAM Module 2 (Egress side)
--    SDR2_ADDR   : OUT   STD_LOGIC_VECTOR(11 DOWNTO 0);
--    SDR2_DATA   : INOUT STD_LOGIC_VECTOR(63 DOWNTO 0);
--    SDR2_RAS    : OUT   STD_LOGIC;        -- Row Address Strobe
--    SDR2_CAS    : OUT   STD_LOGIC;        -- Column Address Strobe
--    SDR2_CE     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- Chip Enable
--    SDR2_BA     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  --
--    SDR2_S      : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- 
--    SDR2_SDA    : OUT   STD_LOGIC;        --
--    SDR2_SDC    : OUT   STD_LOGIC;        --
--    SDR2_WE     : OUT   STD_LOGIC;        -- Write Enable

    -- Test Connector Pins
    RAD_TEST1 : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
    RAD_TEST2 : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);

    -- Test LED Pins
    RAD_LED1 : OUT STD_LOGIC;
    RAD_LED2 : OUT STD_LOGIC;
    RAD_LED3 : OUT STD_LOGIC;
    RAD_LED4 : OUT STD_LOGIC
    );
END rad_loopback;
-------------------------------------------------------------------------------
architecture structure of rad_loopback is
--
-- Component Declarations
--
  COMPONENT IOBUF_F_12
    port (
      O  : out   std_ulogic;
      I  : in    std_ulogic;
      IO : inout std_logic;
      T  : in    std_logic
      );
  end COMPONENT;
--
  COMPONENT BUFGDLL
    port ( O : out std_ulogic;
           I : in  std_ulogic
           );   
  end COMPONENT;
--
-- synthesis translate_off
  for all : IOBUF_F_12 use entity unisim.IOBUF_F_12(IOBUF_F_12_V);
  for all : BUFGDLL use entity unisim.bufgdll(BUFGDLL_V);
-- synthesis translate_on
--
  component rad_loopback_core
    port (
      RAD_CLK      : IN    STD_LOGIC;
      RAD_CLKB     : IN    STD_LOGIC;
      RAD_RESET    : IN    STD_LOGIC;
      RAD_READY    : OUT   STD_LOGIC;
      SOC_LC_NID   : IN    STD_LOGIC;
      D_LC_NID     : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
      TCAFF_LC_RAD : OUT   STD_LOGIC;
      SOC_LC_RAD   : OUT   STD_LOGIC;
      D_LC_RAD     : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0);
      TCAFF_LC_NID : IN    STD_LOGIC;
      SOC_SW_NID   : IN    STD_LOGIC;
      D_SW_NID     : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
      TCAFF_SW_RAD : OUT   STD_LOGIC;
      SOC_SW_RAD   : OUT   STD_LOGIC;
      D_SW_RAD     : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0);
      TCAFF_SW_NID : IN    STD_LOGIC;
      RAD_TEST1    : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
      RAD_TEST2    : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
      RAD_LED1     : OUT   STD_LOGIC;
      RAD_LED2     : OUT   STD_LOGIC;
      RAD_LED3     : OUT   STD_LOGIC;
      RAD_LED4     : OUT   STD_LOGIC);
  end component;
-------------------------------------------------------------------------------
--
-- Signal Declarations
--
  signal rad_clk_dll      : std_logic;  -- DLL clock output
  signal rad_clkb_dll     : std_logic;  -- DLL clock output
--
  signal RAD_RESET_i      : STD_LOGIC;
  signal RAD_READY_i      : STD_LOGIC;
  signal SOC_LC_NID_i     : STD_LOGIC;
  signal D_LC_NID_i       : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal TCAFF_LC_RAD_i   : STD_LOGIC;
  signal SOC_LC_RAD_i     : STD_LOGIC;
  signal D_LC_RAD_i       : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal TCAFF_LC_NID_i   : STD_LOGIC;
  signal SOC_SW_NID_i     : STD_LOGIC;
  signal D_SW_NID_i       : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal TCAFF_SW_RAD_i   : STD_LOGIC;
  signal SOC_SW_RAD_i     : STD_LOGIC;
  signal D_SW_RAD_i       : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal TCAFF_SW_NID_i   : STD_LOGIC;
  signal RAD_TEST1_i      : STD_LOGIC_VECTOR(15 DOWNTO 0);
  signal RAD_TEST2_i      : STD_LOGIC_VECTOR(15 DOWNTO 0);
  signal RAD_LED1_i       : STD_LOGIC;
  signal RAD_LED2_i       : STD_LOGIC;
  signal RAD_LED3_i       : STD_LOGIC;
  signal RAD_LED4_i       : STD_LOGIC;
--
  signal RAD_RESET_pad    : STD_LOGIC;
  signal RAD_READY_pad    : STD_LOGIC;
  signal SOC_LC_NID_pad   : STD_LOGIC;
  signal D_LC_NID_pad     : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal TCAFF_LC_RAD_pad : STD_LOGIC;
  signal SOC_LC_RAD_pad   : STD_LOGIC;
  signal D_LC_RAD_pad     : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal TCAFF_LC_NID_pad : STD_LOGIC;
  signal SOC_SW_NID_pad   : STD_LOGIC;
  signal D_SW_NID_pad     : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal TCAFF_SW_RAD_pad : STD_LOGIC;
  signal SOC_SW_RAD_pad   : STD_LOGIC;
  signal D_SW_RAD_pad     : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal TCAFF_SW_NID_pad : STD_LOGIC;
  signal RAD_TEST1_pad    : STD_LOGIC_VECTOR(15 DOWNTO 0);
  signal RAD_TEST2_pad    : STD_LOGIC_VECTOR(15 DOWNTO 0);
  signal RAD_LED1_pad     : STD_LOGIC;
  signal RAD_LED2_pad     : STD_LOGIC;
  signal RAD_LED3_pad     : STD_LOGIC;
  signal RAD_LED4_pad     : STD_LOGIC;
-------------------------------------------------------------------------------
begin  -- structural
--
-- Instantiate Clock DLL's
--
--  DLL_GCK0 : BUFGDLL
--    PORT MAP (
--      I => rad_clk,
--      O => rad_clk_dll
--      );
  rad_clk_dll  <= rad_clk;
----
--  DLL_GCK1 : BUFGDLL
--    PORT MAP (
--      I => rad_clkb,
--      O => rad_clkb_dll
--      );
--
  rad_clkb_dll <= rad_clkb;

-- Instantiate the SRAM bi-directional data bus
--
--ALL_SRAM_DATA_PAD:
--  for i in 35 downto 0 generate
--    SRAM_DATA_PAD : 
--      IOBUF_F_12 
--        port map ( 
--          O  => sram_data_in(i) , 
--          I  => sram_data_out(i) ,
--          IO => SRAM1_DATA(i),
--          T  => sram_d_dir(i)
--          );
--  end generate ALL_SRAM_DATA_PAD;
--

  -- purpose: Double buffer all off-chip signals
  IOB_Flops : process (rad_clk_dll)
  begin  -- process IOB_Flops
    if rad_clk_dll'event and rad_clk_dll = '1' then  -- rising clock edge
      RAD_RESET_pad    <= RAD_RESET;
      RAD_READY        <= RAD_READY_pad;
      SOC_LC_NID_pad   <= SOC_LC_NID;
      D_LC_NID_pad     <= D_LC_NID;
      TCAFF_LC_RAD     <= TCAFF_LC_RAD_pad;
      SOC_LC_RAD       <= SOC_LC_RAD_pad;
      D_LC_RAD         <= D_LC_RAD_pad;
      TCAFF_LC_NID_pad <= TCAFF_LC_NID;
      SOC_SW_NID_pad   <= SOC_SW_NID;
      D_SW_NID_pad     <= D_SW_NID;
      TCAFF_SW_RAD     <= TCAFF_SW_RAD_pad;
      SOC_SW_RAD       <= SOC_SW_RAD_pad;
      D_SW_RAD         <= D_SW_RAD_pad;
      TCAFF_SW_NID_pad <= TCAFF_SW_NID;
      RAD_TEST1        <= RAD_TEST1_pad;
      RAD_TEST2        <= RAD_TEST2_pad;
      RAD_LED1         <= RAD_LED1_pad;
      RAD_LED2         <= RAD_LED2_pad;
      RAD_LED3         <= RAD_LED3_pad;
      RAD_LED4         <= RAD_LED4_pad;
--
      RAD_RESET_i      <= RAD_RESET_pad;
      RAD_READY_pad    <= RAD_READY_i;
      SOC_LC_NID_i     <= SOC_LC_NID_pad;
      D_LC_NID_i       <= D_LC_NID_pad;
      TCAFF_LC_RAD_pad <= TCAFF_LC_RAD_i;
      SOC_LC_RAD_pad   <= SOC_LC_RAD_i;
      D_LC_RAD_pad     <= D_LC_RAD_i;
      TCAFF_LC_NID_i   <= TCAFF_LC_NID_pad;
      SOC_SW_NID_i     <= SOC_SW_NID_pad;
      D_SW_NID_i       <= D_SW_NID_pad;
      TCAFF_SW_RAD_pad <= TCAFF_SW_RAD_i;
      SOC_SW_RAD_pad   <= SOC_SW_RAD_i;
      D_SW_RAD_pad     <= D_SW_RAD_i;
      TCAFF_SW_NID_i   <= TCAFF_SW_NID_pad;
      RAD_TEST1_pad    <= RAD_TEST1_i;
      RAD_TEST2_pad    <= RAD_TEST2_i;
      RAD_LED1_pad     <= RAD_LED1_i;
      RAD_LED2_pad     <= RAD_LED2_i;
      RAD_LED3_pad     <= RAD_LED3_i;
      RAD_LED4_pad     <= RAD_LED4_i;
    end if;
  end process IOB_Flops;

  rad_loopback_core_1 : rad_loopback_core
    port map (
      RAD_CLK      => rad_clk_dll,
      RAD_CLKB     => rad_clkb_dll,
      RAD_RESET    => RAD_RESET_i,
      RAD_READY    => RAD_READY_i,
      SOC_LC_NID   => SOC_LC_NID_i,
      D_LC_NID     => D_LC_NID_i,
      TCAFF_LC_RAD => TCAFF_LC_RAD_i,
      SOC_LC_RAD   => SOC_LC_RAD_i,
      D_LC_RAD     => D_LC_RAD_i,
      TCAFF_LC_NID => TCAFF_LC_NID_i,
      SOC_SW_NID   => SOC_SW_NID_i,
      D_SW_NID     => D_SW_NID_i,
      TCAFF_SW_RAD => TCAFF_SW_RAD_i,
      SOC_SW_RAD   => SOC_SW_RAD_i,
      D_SW_RAD     => D_SW_RAD_i,
      TCAFF_SW_NID => TCAFF_SW_NID_i,
      RAD_TEST1    => RAD_TEST1_i,
      RAD_TEST2    => RAD_TEST2_i,
      RAD_LED1     => RAD_LED1_i,
      RAD_LED2     => RAD_LED2_i,
      RAD_LED3     => RAD_LED3_i,
      RAD_LED4     => RAD_LED4_i);
end structure;

-- synthesis translate_off
-- configuration rad_loopback_conf of rad_loopback is
--   for structure
--     for all : rad_loopback_core
--       use configuration work.rad_loopback_core_conf;
--     end for;
--   end for;
-- end rad_loopback_conf;
--synthesis translate_on
