--  Applied Research Laboratory
--  Washington University in St. Louis
--
--  File:  rad_loopback_core.vhd
--  Top level structure for RAD FPGA with ingress/egress loopback modules
--  Created by:  John W. Lockwood (lockwood@arl.wustl.edu),
--  David E. Taylor (det3@arl.wustl.edu)
--
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
--library unisim;
--
ENTITY rad_loopback_core IS
  PORT (
    -- Clocks
    RAD_CLK  : IN STD_LOGIC;
    RAD_CLKB : IN STD_LOGIC;

    -- Reset & Reconfig
    RAD_RESET : IN STD_LOGIC;
    RAD_READY : OUT STD_LOGIC;
--    RAD_RECONFIG : INOUT STD_LOGIC_VECTOR(2 DOWNTO 0);

    -- NID Interface
    --
    -- Ingress Path
    -- Input
    SOC_LC_NID   : IN  STD_LOGIC;
    D_LC_NID     : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    TCAFF_LC_RAD : OUT STD_LOGIC;
    -- Output
    SOC_LC_RAD   : OUT STD_LOGIC;
    D_LC_RAD     : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    TCAFF_LC_NID : IN  STD_LOGIC;

    -- Egress Path
    -- Input
    SOC_SW_NID   : IN  STD_LOGIC;
    D_SW_NID     : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    TCAFF_SW_RAD : OUT STD_LOGIC;
    -- Output
    SOC_SW_RAD   : OUT STD_LOGIC;
    D_SW_RAD     : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    TCAFF_SW_NID : IN  STD_LOGIC;

    -- SRAM Module 1 (Ingress Side)
--    SRAM1_ADDR  : OUT   STD_LOGIC_VECTOR(17 DOWNTO 0);
--    SRAM1_DATA  : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
--    SRAM1_CE    : OUT   STD_LOGIC;        -- Chip Enable: LOW=drive
--    SRAM1_GW    : OUT   STD_LOGIC;        -- Global Write: LOW=write
--    SRAM1_PIN84 : OUT   STD_LOGIC;

    -- SRAM Module 2 (Egress side)
--    SRAM2_ADDR  : OUT   STD_LOGIC_VECTOR(17 DOWNTO 0);
--    SRAM2_DATA  : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
--    SRAM2_CE    : OUT   STD_LOGIC;        -- Chip Enable: LOW=drive
--    SRAM2_GW    : OUT   STD_LOGIC;        -- Global Write: LOW=write
--    SRAM2_PIN84 : OUT   STD_LOGIC;

    -- SDRAM Module 1 (Ingress side)
--    SDR1_ADDR   : OUT   STD_LOGIC_VECTOR(11 DOWNTO 0);
--    SDR1_DATA   : INOUT STD_LOGIC_VECTOR(63 DOWNTO 0);
--    SDR1_RAS    : OUT   STD_LOGIC;        -- Row Address Strobe
--    SDR1_CAS    : OUT   STD_LOGIC;        -- Column Address Strobe
--    SDR1_CE     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- Chip Enable
--    SDR1_BA     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  --
--    SDR1_S      : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- 
--    SDR1_SDA    : OUT   STD_LOGIC;        --
--    SDR1_SDC    : OUT   STD_LOGIC;        --
--    SDR1_WE     : OUT   STD_LOGIC;        -- Write Enable

    -- SDRAM Module 2 (Egress side)
--    SDR2_ADDR   : OUT   STD_LOGIC_VECTOR(11 DOWNTO 0);
--    SDR2_DATA   : INOUT STD_LOGIC_VECTOR(63 DOWNTO 0);
--    SDR2_RAS    : OUT   STD_LOGIC;        -- Row Address Strobe
--    SDR2_CAS    : OUT   STD_LOGIC;        -- Column Address Strobe
--    SDR2_CE     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- Chip Enable
--    SDR2_BA     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  --
--    SDR2_S      : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- 
--    SDR2_SDA    : OUT   STD_LOGIC;        --
--    SDR2_SDC    : OUT   STD_LOGIC;        --
--    SDR2_WE     : OUT   STD_LOGIC;        -- Write Enable

    -- Test Connector Pins
--    RAD_TEST1 : INOUT STD_LOGIC_VECTOR(15 DOWNTO 0);
--    RAD_TEST2 : INOUT STD_LOGIC_VECTOR(15 DOWNTO 0);
    RAD_TEST1 : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
    RAD_TEST2 : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);

    -- Test LED Pins
    RAD_LED1 : OUT STD_LOGIC;
    RAD_LED2 : OUT STD_LOGIC;
    RAD_LED3 : OUT STD_LOGIC;
    RAD_LED4 : OUT STD_LOGIC
    );
END rad_loopback_core;
-------------------------------------------------------------------------------
architecture structure of rad_loopback_core is
--
-- Component Declarations
--
  component loopback_module
    port (
      clk         : in  STD_LOGIC;
      reset_l     : in  STD_LOGIC;
      soc_mod_in  : in  STD_LOGIC;
      d_mod_in    : in  STD_LOGIC_VECTOR(31 downto 0);
      tca_mod_in  : out STD_LOGIC;
      soc_out_mod : out STD_LOGIC;
      d_out_mod   : out STD_LOGIC_VECTOR(31 downto 0);
      tca_out_mod : in  STD_LOGIC;
      test_data   : out STD_LOGIC_VECTOR(31 downto 0)
      );
  end component;
--
  component InterfaceModule
    port (
      clk         : in  STD_LOGIC;
      reset_l     : in  STD_LOGIC;
      soc_mod_in  : in  STD_LOGIC;
      d_mod_in    : in  STD_LOGIC_VECTOR(31 downto 0);
      tca_mod_in  : out STD_LOGIC;
      soc_out_mod : out STD_LOGIC;
      d_out_mod   : out STD_LOGIC_VECTOR(31 downto 0);
      tca_out_mod : in  STD_LOGIC;
	  enable_l    : in  STD_LOGIC;
	  ready_l     : out STD_LOGIC;
      test_data   : out STD_LOGIC_VECTOR(31 downto 0)
      );
  end component;
--
  component blink
    port (
      clk1    : IN  STD_LOGIC;
      clk2    : IN  STD_LOGIC;
      reset_l : IN  STD_LOGIC;
      led1    : OUT STD_LOGIC;
      led2    : OUT STD_LOGIC);
  end component;
--
-- Signal Declarations
--
  signal ingress_test, egress_test : std_logic_vector(31 downto 0); -- test pin data
  signal logic0, logic1 : std_logic; -- dummy
--
begin  -- structural
--
  rad_ready <= NOT(rad_reset);
--  rad_reconfig(2) <= NOT(rad_reset);
--
-- Test Pin Flops
  TEST_PIN_FF : process (RAD_CLK)
  begin  -- process TEST_PIN_FF
    if RAD_CLK'event and RAD_CLK = '1' then  -- rising clock edge
        rad_test2 <= ingress_test(31 downto 16);
        rad_test1 <= ingress_test(15 downto 0);
--
        rad_led3 <= rad_reset;
        rad_led4 <= not rad_reset;
    end if;
  end process TEST_PIN_FF;
--      
  logic0 <= '0';
  logic1 <= '1';
-------------------------------------------------------------------------------
  INGRESS : InterfaceModule 
    port map (
      clk         => RAD_CLK,
      reset_l     => rad_reset,
      soc_mod_in  => soc_lc_nid,
      d_mod_in    => d_lc_nid,
      tca_mod_in  => tcaff_lc_rad,
      soc_out_mod => soc_lc_rad,
      d_out_mod   => d_lc_rad,
      tca_out_mod => tcaff_lc_nid,
      enable_l    => logic0,
      ready_l     => open,
      test_data   => ingress_test);
--
  EGRESS : loopback_module
    port map (
      clk         => RAD_CLKB,
      reset_l     => rad_reset,
      soc_mod_in  => soc_sw_nid,
      d_mod_in    => d_sw_nid,
      tca_mod_in  => tcaff_sw_rad,
      soc_out_mod => soc_sw_rad,
      d_out_mod   => d_sw_rad,
      tca_out_mod => tcaff_sw_nid,
      test_data   => egress_test);
--
  BLINK1 : blink
    port map (
      clk1    => RAD_CLK,
      clk2    => RAD_CLKB,
      reset_l => rad_reset,
      led1    => rad_led1,
      led2    => rad_led2);
-------------------------------------------------------------------------------  

end structure;

-- synthesis translate_off
-- configuration rad_loopback_core_conf of rad_loopback_core is
--   for structure
--     for all : InterfaceModule
--       use configuration work.Interface_conf;
--     end for;
--   end for;
-- end rad_loopback_core_conf;
--synthesis translate_on
