/* $Id: ctrlcell.c,v 1.1 2001/04/06 22:00:18 florian Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * ctrlcell.c: convert control cell blocks to atm cells
 */

#include "testbench.h"
#include <stdlib.h>
#include <string.h>

#define MIN(a,b) ( (a)<(b) ? (a) : (b))

extern void set_cc_crc (DataBlock*data);

StreamBlock *CtrlCellFilter (StreamBlock *block)
{
  DataBlock *data;
  unsigned opcode = 0;
  unsigned modid = 0;
  unsigned vci = 0x23;
  unsigned crc = 0;
  unsigned plnum;
  unsigned count;
  char strbuf[21];
  static unsigned seqnr = 0;

  /* we only handle control cell blocks */
  if (block->btype != CTRLCELL)
    return block;
  data = NewDataBlock ();

  /* check options */
  if (block->options) {
    sscanf (block->options, "%d %d %d", &opcode, &modid, &vci);
    free (block->options);
    block->options = NULL;
  }

  /* control cell header */
  AddToDataBlock (data, (opcode << 24) | (modid << 16) );
  
  /* copy payload */
  plnum = MIN(block->data->entries, 9);/* min entries, 9 */
  for (count=0; count<plnum; count++)
    AddToDataBlock (data, block->data->data[count]);

  /* pad to 9 words */
  for (; count < 9; count++)
    AddToDataBlock (data, 0L);

  /* control cell trailer */
  AddToDataBlock (data, 0L);	/* cm data */
  AddToDataBlock (data, seqnr << 16);
  seqnr ++;
  /* set crc */
  set_cc_crc (data);

  /* delete old block */
  DeleteDataBlock (block->data);

  /* update stream block */
  block->btype = CELL;
  block->data = data;
  sprintf (strbuf, "%d", vci);
  block->options = strdup (strbuf);

  return block;
}
