/* $Id: dumpnidsim.c,v 1.1 2001/07/24 01:41:59 florian Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpattern for FPX modules
 *
 * dumpnidsim.c: output macro file for vsim, signals for NID
 */

#include "testbench.h"
#include <stdio.h>

void DumpNIDSim (char *outfile, StreamBlock *sblock)
{
  FILE *file;
  unsigned count;

  /* open file */
  file = fopen (outfile, "w");
  if (!file) {
    printf ("Error! cannot open file %s\n", outfile);
    return;
  }

  /* dump raw blocks */
  while (sblock) {
    unsigned pad = 0;
    unsigned framevci = 50;

    switch (sblock->btype) {
    case PAD:
      if (sblock->options)
	sscanf (sblock->options, "%d", &pad);
      fprintf (file, "run %d\n", 10*pad);
      break;
    case RAW:
      /* first cell word */
      fprintf (file, "force soc_sw_opp 1\n");
      if (sblock->data->entries >= 1) {
	fprintf (file, "force d_h_sw_opp X\"%04x\"\n", 
		 sblock->data->data[0] >> 16);
	fprintf (file, "force d_l_sw_opp X\"%04x\"\n", 
		 sblock->data->data[0] & 0xFFFF);

      }
      fprintf (file, "run 10\n");
      /* second cell word */
      fprintf (file, "force soc_sw_opp 0\n");
      if (sblock->data->entries >= 2) {
	fprintf (file, "force d_h_sw_opp X\"%04x\"\n", 
		 sblock->data->data[1] >> 16);
	fprintf (file, "force d_l_sw_opp X\"%04x\"\n", 
		 sblock->data->data[1] & 0xFFFF);
      }
      fprintf (file, "run 10\n");
      /* dump data */
      for (count=2; count<sblock->data->entries; count++) {
	fprintf (file, "force d_h_sw_opp X\"%04x\"\n", 
		 sblock->data->data[count] >> 16);
	fprintf (file, "force d_l_sw_opp X\"%04x\"\n", 
		 sblock->data->data[count] & 0xFFFF);
	fprintf (file, "run 10\n");
      }
      break;
    case FRAME:
      /* no frame level for NID */
      break;
    } /* switch block-type */

    /* get next block */
    sblock = sblock->next;
  }

  /* close file */
  fclose (file);
}
