/* $Id: dumpsim.c,v 1.4 2001/07/24 01:42:43 florian Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpattern for FPX modules
 *
 * dumpsim.c: output macro file for vsim
 */

#include "testbench.h"
#include <stdio.h>

void DumpSim (char *outfile, StreamBlock *sblock)
{
  FILE *file;
  unsigned count;

  /* open file */
  file = fopen (outfile, "w");
  if (!file) {
    printf ("Error! cannot open file %s\n", outfile);
    return;
  }

  /* dump raw blocks */
  while (sblock) {
    unsigned pad = 0;
    unsigned framevci = 50;

    switch (sblock->btype) {
    case PAD:
      if (sblock->options)
	sscanf (sblock->options, "%d", &pad);
      fprintf (file, "run %d\n", 10*pad);
      break;
    case RAW:
      /* first cell word */
      fprintf (file, "force soc_mod_in 1\n");
      if (sblock->data->entries >= 1)
	fprintf (file, "force d_mod_in X\"%08x\"\n", sblock->data->data[0]);
      fprintf (file, "run 10\n");
      /* second cell word */
      fprintf (file, "force soc_mod_in 0\n");
      if (sblock->data->entries >= 2)
	fprintf (file, "force d_mod_in X\"%08x\"\n", sblock->data->data[1]);
      fprintf (file, "run 10\n");
      /* dump data */
      for (count=2; count<sblock->data->entries; count++) {
	fprintf (file, "force d_mod_in X\"%08x\"\n", 
		 sblock->data->data[count]);
	fprintf (file, "run 10\n");
      }
      break;
    case FRAME:
      /* options */
      if (sblock->options)
	sscanf (sblock->options, "%d", &framevci);
      /* header */
      fprintf (file, "force sof_mod_in 1\n");
      fprintf (file, "force d_mod_in X\"%08x\"\n", framevci<<4);
      fprintf (file, "run 10\n");
      fprintf (file, "force sof_mod_in 0\n");
      fprintf (file, "force dataen_mod_in 1\n");

      /* frame */
      for (count=0; count<sblock->data->entries; count++) {
	if (count+1 == sblock->data->entries)
	  fprintf (file, "force eof_mod_in 1\n");

	/* payload */
	fprintf (file, "force d_mod_in X\"%08x\"\n", 
		 sblock->data->data[count]);
	fprintf (file, "run 10\n");
      }
      fprintf (file, "force eof_mod_in 0\n");
      /* trailer */
      fprintf (file, "force d_mod_in X\"%08x\"\n",
	       sblock->data->entries << 2);
      fprintf (file, "run 10\n");
      fprintf (file, "force d_mod_in X\"00000000\"\n");	/* assume no error */
      fprintf (file, "run 10\n");
      fprintf (file, "force dataen_mod_in 0\n");
      break;
    case UDP:
      /* options */
      if (sblock->options)
	sscanf (sblock->options, "%d", &framevci);
      /* header */
      fprintf (file, "force sof_mod_in 1\n");
      fprintf (file, "force d_mod_in X\"%08x\"\n", framevci<<4);
      fprintf (file, "run 10\n");
      fprintf (file, "force sof_mod_in 0\n");
      fprintf (file, "force dataen_mod_in 1\n");

      /* frame */
      for (count=0; count<sblock->data->entries; count++) {
	if (count+1 == sblock->data->entries)
	  fprintf (file, "force eof_mod_in 1\n");

	/* payload */
	fprintf (file, "force d_mod_in X\"%08x\"\n", 
		 sblock->data->data[count]);
	fprintf (file, "run 10\n");
      }
      fprintf (file, "force eof_mod_in 0\n");
      /* trailer */
      fprintf (file, "force d_mod_in X\"%08x\"\n",
	       sblock->data->entries << 2);
      fprintf (file, "run 10\n");
      fprintf (file, "force d_mod_in X\"00000000\"\n");	/* assume no error */
      fprintf (file, "run 10\n");
      fprintf (file, "force dataen_mod_in 0\n");
      break;
    } /* switch block-type */

    /* get next block */
    sblock = sblock->next;
  }

  /* close file */
  fclose (file);
}
