/* $Id: hec.c,v 1.3 2001/07/24 01:56:32 florian Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * hec.c: code to calculate the ATM HEC. Original code from 
 *        http://cell-relay.indiana.edu/cell-relay/publications/software/
 *               CRC/32bitCRC.tutorial.html
 */

#include <sys/types.h>
#include <netinet/in.h>

#define HEC_GENERATOR   0x107               /* x^8 + x^2 +  x  + 1  */
#define COSET_LEADER    0x055               /* x^6 + x^4 + x^2 + 1  */

static unsigned char syndrome_table[256];
static int hec_defined = 0;

static void gen_syndrome_table()
/* generate a table of CRC-8 syndromes for all possible input bytes */
{
    register int i, j, syndrome;
    for ( i = 0;  i < 256;  i++ )
    {
        syndrome = i;
        for ( j = 0;  j < 8;  j++ )
        {
            if ( syndrome & 0x80 )
               syndrome = ( syndrome << 1 ) ^ HEC_GENERATOR;
            else
               syndrome = ( syndrome << 1 );
        }
        syndrome_table[i] = (unsigned char) syndrome;
    }
    
    hec_defined = 1;
    return;
}

void insert_hec(cell_header)
unsigned char cell_header[5];
/* calculate CRC-8 remainder over first four bytes of cell header   */
/* exclusive-or with coset leader & insert into fifth header byte   */
{
    register unsigned char hec_accum = 0;
    register int i;
    for ( i = 0;  i < 4;  i++ )
        hec_accum = syndrome_table [ hec_accum ^ cell_header[i] ];
    cell_header [4] = hec_accum ^ COSET_LEADER;
    return;
}

unsigned char calc_hec (unsigned long header)
{
  unsigned char atm_hdr[5];
  
  if (!hec_defined)
    gen_syndrome_table();

  *((long*)atm_hdr) = htonl (header);
  atm_hdr[4] = 0;
  insert_hec (atm_hdr);

  return atm_hdr[4];
}
