/* $Id: ipv4.c,v 1.1.1.1 2001/03/21 20:11:36 florian Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 * 
 * ipv4.c: converts IPv4 blocks to frame blocks
 */


#include "testbench.h"
#include <stdlib.h>
#include <string.h>

StreamBlock *IPv4Filter (StreamBlock *block)
{
  DataBlock *data = NewDataBlock ();
  char strbuf[33];
  unsigned addr[4] = { 127, 0, 0, 1};
  unsigned vci = 0x32;
  unsigned proto = 1;
  unsigned ttl = 64;
  unsigned hcs;
  unsigned count;

  /* we only handle IPv4 blocks */
  if (block->btype != IPv4)
    return block;

  /* check options */
  if (block->options) {
    sscanf (block->options, "%d.%d.%d.%d %d %d %d", 
	    addr, addr+1, addr+2, addr+3, &proto, &ttl, &vci);
    free (block->options);
  }

  /* set options */
  sprintf (strbuf, "%d", vci);
  block->options = strdup (strbuf);

  /* create IP header */
  /* version 4, headerlength 5, no special type of service, length) */
  AddToDataBlock (data, 0x45000000L | (block->data->entries*4 + 20));
  /* id , flags and offset */
  AddToDataBlock (data, 0L);
  /* ttl, protocol and checksum(later) */
  AddToDataBlock (data, (ttl << 24) | (proto << 16) );
  /* source address == 127.0.0.1 */
  AddToDataBlock (data, 0x7F000001);
  /* destination address */
  AddToDataBlock (data, (addr[0]<<24)|(addr[1]<<16)|(addr[2]<<8)|addr[3]);

  /* calc header checksum */
  for (hcs=0, count=0; count<5; count++)
    hcs += (data->data[count] >> 16) + (data->data[count]&0xFFFF);
  while (hcs & 0xFFFF0000)
    hcs = (hcs & 0xFFFF) + (hcs >> 16);
  data->data[2] |= (~hcs & 0xFFFF);

  /* copy payload */
  for (count=0; count<block->data->entries; count++)
    AddToDataBlock (data, block->data->data[count]);

  /* delete old block */
  DeleteDataBlock (block->data);
  
  /* update stream block */
  block->btype = FRAME;
  block->data = data;

  return block;
}
