/* $Id: ipv6.c,v 1.1.1.1 2001/03/21 20:11:36 florian Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * ipv6.c: converts IPv6 blocks to frame blocks
 */

#include "testbench.h"
#include <stdlib.h>
#include <string.h>

StreamBlock *IPv6Filter (StreamBlock *block)
{
  DataBlock *data = NewDataBlock ();
  char strbuf[33];
  unsigned addr[8] = { 0, 0, 0, 0, 0, 0, 0, 1};
  unsigned vci = 1;
  unsigned ttl = 255;
  unsigned count;

  /* we only handle IPv6 blocks */
  if (block->btype != IPv6)
    return block;

  /* check options */
  if (block->options) {
    sscanf (block->options, "%x:%x:%x:%x:%x:%x:%x:%x %d %d", 
	    addr, addr+1, addr+2, addr+3, addr+4, addr+5, addr+6, addr+7,
	    &ttl, &vci);
    free (block->options);
  }

  /* set options */
  sprintf (strbuf, "%d", vci);
  block->options = strdup (strbuf);

  /* create IP header */
  /* version 6, priority 0, flowlabel 0 */
  AddToDataBlock (data, 0x60000000L);
  /* payload length, next header, ttl */
  AddToDataBlock (data, (block->data->entries*4)<<16 | ttl);
  /* src address ::1 */
  AddToDataBlock (data, 0L);
  AddToDataBlock (data, 0L);
  AddToDataBlock (data, 0L);
  AddToDataBlock (data, 1L);
  /* destination address */
  AddToDataBlock (data, (addr[0]<<16)|(addr[1]) );
  AddToDataBlock (data, (addr[2]<<16)|(addr[3]) );
  AddToDataBlock (data, (addr[4]<<16)|(addr[5]) );
  AddToDataBlock (data, (addr[6]<<16)|(addr[7]) );

  /* copy payload */
  for (count=0; count<block->data->entries; count++)
    AddToDataBlock (data, block->data->data[count]);

  /* delete old block */
  DeleteDataBlock (block->data);
  
  /* update stream block */
  block->btype = FRAME;
  block->data = data;

  return block;
}
