/* $Id: parser.c,v 1.2 2001/04/06 21:59:58 florian Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * parser.c: parse description file
 */

#include "testbench.h"
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <netinet/in.h>

/* commands */
struct {
  char *cmd;
  BlockType btype;
} commands[] = {
  { "RAW", RAW},
  { "PAD", PAD},
  { "CELL", CELL},
  { "CTRLCELL", CTRLCELL},
  { "FRAME", FRAME},
  { "IPv4", IPv4},
  { "IPv6", IPv6},
  { "UDP", UDP},
  { "ERROR", ERROR},
};
#define NRCOMMANDS (sizeof(commands)/sizeof(commands[0]))

StreamBlock *ParseFile (char *filename)
{
  FILE *file;
  char line[81], *options, *p;
  StreamBlock *start, *sblock;
  unsigned idx, word;

  /* open file */
  file = fopen (filename, "r");
  if (!file) {
    printf ("error opening file %s\n", filename);
    return NULL;
  }

  /* create new stream block */
  start = sblock = NewStreamBlock (NONE);

  /* parse file */
  while (!feof(file)) {
    /* read line */
    if (fgets (line, 80, file) ) {
      /* read ok */
      if (line[0] == '\0' || line[0] == '\n' || line[0] == '\r')
	continue;		/* empty line */
      if (line[0] == '#')	
	continue;		/* comment */
      
      if (line[0] == '!') {	/* command */
	/* trunc command */
	options = index(line, ' ');
	if (!options) options = index(line, '\t');
	if (!options) options = index(line, '\n');
	if (!options) options = index(line, '\r');

	if (options) *options++ = '\0';

	/* cut trailing newline */
	if (p = index (options, '\n')) *p='\0';
	if (p = index (options, '\r')) *p='\0';

	/* find command */
	for (idx=0; idx<NRCOMMANDS; idx++) 
	  if (!strcmp (commands[idx].cmd, line+1))
	    break;
	if (idx >= NRCOMMANDS) 
	  printf ("Warning! unknown command %s\n", line);
	else {
	  /* create new block for command */
	  sblock->next = NewStreamBlock (commands[idx].btype);
	  sblock = sblock->next;
	  if (*options)
	    sblock->options = strdup (options);
	}
      }	/* is command */
      else {
	/* parse number */
	sscanf (line, "%x", &word);
	/* add word to block */
	AddToDataBlock (sblock->data, word);
	
      } /* parse number */
    } /* read line */

  } /* not end of file */

  /* close file */
  fclose (file);

  /* return stream */
  return start;
}
