/* $Id: raipv4.c,v 1.1.1.1 2001/03/21 20:11:36 florian Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 * 
 * raipv4.c: reassemble ipv4 packets from frames
 */


#include "testbench.h"
#include <stdlib.h>
#include <string.h>

#define VCI 1

StreamBlock *RAIPv4Filter (StreamBlock *block)
{
  DataBlock *data;
  char strbuf[33];
  unsigned vci = VCI;
  unsigned hcs;
  unsigned count;

  /* we only handle FRAME blocks */
  if (block->btype != FRAME)
    return block;

  /* get options */
  if (block->options)
    sscanf (block->options, "%d", &vci);

  if (vci != VCI)		/* ignore vci's other than default */
    return block;

  /* check header (must be version 4) */
  if ((block->data->entries < 1) 
      || (block->data->data[0] >> 28 != 4)) 
    return block;

  /* check header size */
  if (block->data->entries < 5) {
    printf ("incorrect header size!\n");
    DeleteStreamBlock (block);
    return ErrorBlock ("IPv4 header too short");
  }

  /* check header checksum */
  for (hcs=0, count=0; count<5; count++)
    hcs += (block->data->data[count] >> 16) 
            + (block->data->data[count]&0xFFFF);
  while (hcs & 0xFFFF0000)
    hcs = (hcs & 0xFFFF) + (hcs >> 16);
  
  if (hcs != 0xFFFF) {
    /* checksum not correct */
    printf ("invalid header checksum!\n");
    DeleteStreamBlock (block);
    return ErrorBlock ("invalid header checksum in IPv4 header");
  }

  /* set options (ip, proto, ttl and vci) */
  if (block->options) 
    free (block->options);
  sprintf (strbuf, "%d.%d.%d.%d %d %d %d",
	   (block->data->data[4]>>24) & 0xFF,
	   (block->data->data[4]>>16) & 0xFF,
	   (block->data->data[4]>>8)  & 0xFF,
	    block->data->data[4]      & 0xFF,
	   (block->data->data[2]>>16) & 0xFF, 
	   block->data->data[2] >> 24,
	   vci);
  block->options = strdup (strbuf);

  /* copy payload */
  data = NewDataBlock ();
  for (count=5; count<block->data->entries; count++)
    AddToDataBlock (data, block->data->data[count]);

  /* delete old block */
  DeleteDataBlock (block->data);
  
  /* update stream block */
  block->btype = IPv4;
  block->data = data;

  return block;
}
