/* $Id: raudp.c,v 1.2 2001/07/24 01:42:43 florian Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 * 
 * raudp.c: reassembles UDP datagrams from IP packets
 */


#include "testbench.h"
#include <stdlib.h>
#include <string.h>

StreamBlock *RAUDPFilter (StreamBlock *block)
{
  DataBlock *data = NewDataBlock ();
  char strbuf[33];
  unsigned count;
  unsigned addr[4] = { 127, 0, 0, 1};
  unsigned proto = 1;
  unsigned srcport;
  unsigned dstport;
  unsigned length;

  /* we only handle IPv4 blocks */
  if (block->btype != IPv4)
    return block;

  /* check options */
  if (block->options) {
    sscanf (block->options, "%d.%d.%d.%d %d", 
	    addr, addr+1, addr+2, addr+3, &proto);
  }

  /* check protocoll */
  if (proto != 17)
    return block;

  /* check length */
  if (block->data->entries < 3) {
    printf ("incorrect UDP header size!\n");
    DeleteStreamBlock (block);
    return ErrorBlock ("UDP header too short");
  }
  length = block->data->data[1] >> 16;
  printf ("lenght: %d, entries: %d\n", length, block->data->entries);
  if (length/4 > block->data->entries) {
    printf ("UDP datagram truncated!\n");
    DeleteStreamBlock (block);
    return ErrorBlock ("UDP datagram truncated");
  }

  /* check checksum */
  /* NEED SECOND IP ADDRESS! */

  /* src port / dest port */
  srcport = block->data->data[0] >> 16;
  dstport = block->data->data[0] & 0xFFFF;

  /* copy payload */
  for (count=2; count<block->data->entries; count++)
    AddToDataBlock (data, block->data->data[count]);

  /* delete old block */
  DeleteDataBlock (block->data);
  
  /* set options */
  if (block->options)
    free (block->options);
  sprintf (strbuf, "%d.%d.%d.%d %d %d", 
	   addr[0], addr[1], addr[2], addr[3], dstport, srcport);
  block->options = strdup (strbuf);

  /* update stream block */
  block->btype = UDP;
  block->data = data;

  return block;
}
