/* $Id: ttldec.c,v 1.1.1.1 2001/03/21 20:11:36 florian Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * ttldec.c: decrement ttl in ip header, update aal5 crc, works on cells
 */

#include "testbench.h"
#include <stdlib.h>

#define VCI          1

#define FIRST_HEADER 0
#define UPDATE       1
#define IGNORE       2

static int state = FIRST_HEADER;

static unsigned long crcupdate;

extern int tabs_initialized;
extern unsigned long crc_ttl_tab[256];
extern unsigned long crc_hcslo_tab[256];
extern unsigned long crc_hcshi_tab[256];
extern unsigned long crc_inc_tab[4][256];

extern void gen_tabs();

StreamBlock *TTLDecFilter (StreamBlock *block)
{
  unsigned vci = 1;
  unsigned islast = 0;
  unsigned ipversion;
  unsigned count;

  /* ignore blocks other than cells */
  if (block->btype != CELL)
    return block;

  /* get options */
  if (block->options)
    sscanf (block->options, "%d %d", &vci, &islast);
  
  /* ignore vci's other than VCI */
  if (vci != VCI)
    return block;

  switch (state) 
    {
    case FIRST_HEADER:
      /* correct header has at least 7 entries */
      if (block->data->entries < 7) {
	if (islast)
	  state = FIRST_HEADER;
	else 
	  state = IGNORE;
	return block;
      }

      /* check version */
      ipversion = block->data->data[0] >> 28;
      if (ipversion == 4) {
	unsigned long update = block->data->data[2];  
	unsigned ttl = update >> 24;
	unsigned checksum = update & 0xFFFF;

	if (ttl) {
	  ttl = ttl - 1;
	  checksum = ~(~checksum + 0xFEFF);
	  checksum = (checksum & 0xFFFF) + (checksum >> 16);
	  
	  update &= ~0xFF00FFFF;
	  update |= ttl << 24;
	  update |= checksum & 0xFFFF;
	}
	
	/* update cell data */
	update ^= block->data->data[2];
	block->data->data[2] ^= update;

	/* update crc */
	/*	crcupdate = update_crc4 (0x0L, 0x0L);
		crcupdate = update_crc4 (crcupdate, 0L);*/
printf ("update: 0x%x\n", update);
	if (!tabs_initialized)
	  gen_tabs();
	crcupdate = crc_ttl_tab[update>>24]
	           ^crc_hcshi_tab[ (update>>8) & 0xFF]
	           ^crc_hcslo_tab[update & 0xFF];
printf ("update2: 0x%x\n", crcupdate);

	state = UPDATE;
      }
      else if (ipversion == 6) {
	unsigned ttl = block->data->data[1] & 0xFF;
	if (ttl) {
	  /* decrement ttl field */
	  block->data->data[1] &= ~0xFF;
	  block->data->data[1] |= --ttl;
	}
	state = UPDATE;
      }
      else
	state = IGNORE;

      break;

    case UPDATE:
      crcupdate =  crc_inc_tab[0][ crcupdate & 0xFF]
	         ^ crc_inc_tab[1][ (crcupdate >> 8) & 0xFF]
	         ^ crc_inc_tab[2][ (crcupdate >> 16) & 0xFF]
	         ^ crc_inc_tab[3][ (crcupdate >> 24) & 0xFF];
      break;
      
    case IGNORE:
      break;
      
    default:
      state = IGNORE;
    }
  
  /* is last cell? */
  if (islast) {
    /* update crc field */
    /*    crcupdate ^= 0xFFFFFFFF;*/
printf ("crcupdate: 0x%x\n", crcupdate);
    block->data->data[block->data->entries-1] ^= crcupdate;

    state = FIRST_HEADER;
  }
  
  return block;    
}

main (int argc, char**argv)
{
  char *infile = "TBRAW.TBP";
  char *outfile = "TTLDEC.TBP";
  StreamBlock *sblock;

  if (argc > 1)
    infile = argv[1];
  if (argc > 2)
    outfile = argv[2];

  sblock = ParseFile (infile);

  sblock = FilterStream (sblock, RACellFilter);
  sblock = FilterStream (sblock, TTLDecFilter);
  sblock = FilterStream (sblock, CellFilter);

  DumpTB (outfile, sblock); 

  return 0;
}
