/* A C program used to send and receive a UDP datagram */
/* There is an optional verbose mode [-v] */
/* By Henry Fu */
 
#include <stdio.h> 
#include <stdlib.h> 
#include <errno.h> 
#include <string.h> 
#include <netdb.h> 
#include <sys/types.h> 
#include <netinet/in.h> 
#include <sys/socket.h> 

#define MYPORT 3490    /* the source port */
#define TOPORT 8128    /* the destination port */ 
#define MYSIZE 1536    /* the deault datagram size will be 1536 bytes */

/* Globals for getting command line arguments */
static char *options = "p:v";
static char *usage = "[-p port] [-v]";
int vflg = 0, errflg = 0;
int port = TOPORT;
int myport = MYPORT;
int bufsize = MYSIZE;
int size;
unsigned char *buf, *rbuf, *lbuf;
int i;
int x0, x1, x2, x3, y0, y1, y2, y3;

/* Required by getopt */
extern char *optarg;
extern int optind, opterr;

/* Function to get command line arguments */
void parseargs(int argc, char *argv[])
{
    int c;
    int ok = 1;

    while ((c=getopt(argc,argv,options)) != -1)
        switch (c)
        {
        case 'p':
          myport = atoi(optarg);
          break;
        case 'v':
          vflg++;
          break;
        default:
          ok = 0;
        }
   
    if (!ok || optind < argc) {
      fprintf (stderr, "usage : %s %s\n", argv[0], usage);
      exit(0);
    }
}

int main(int argc, char *argv[])
{
    int sockfd;  
    struct hostent *he;
    struct sockaddr_in my_addr;
    struct sockaddr_in their_addr; /* connector's address information */

    buf = calloc(bufsize, sizeof(char));
    rbuf = calloc(bufsize, sizeof(char));
    lbuf = calloc(80, sizeof(char));

    parseargs(argc, argv);

    if (vflg) fprintf (stdout, "Recv is allocating socket ...\n");

    if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
        perror("Recv failed at socket");
        exit(1);
    }

    my_addr.sin_family = AF_INET;
    my_addr.sin_port = htons(myport);
    my_addr.sin_addr.s_addr = INADDR_ANY;
    bzero(&(my_addr.sin_zero), 8);

    if (vflg) fprintf (stdout, "Recv is binding ...\n");

    if (bind(sockfd, (struct sockaddr *)&my_addr, sizeof(struct sockaddr)) 
                                                                  == -1) {
        perror("Recv failed at bind");
        exit(1);
    }

    if (vflg) fprintf (stdout, "Recv is receiving ...\n");

    if ((size = read(sockfd, rbuf, bufsize)) < 0) {
        perror("Recv failed at read");
        exit(1);
    }

    fprintf (stdout, "Received: \n");
    for(i=0; i<size; i+=4) {
      y0 = rbuf[i];
      y1 = rbuf[i+1];
      y2 = rbuf[i+2];
      y3 = rbuf[i+3];
      fprintf (stdout, "%.2X%.2X%.2X%.2X\n",y0,y1,y2,y3);
    }

    close(sockfd);

    return 0;
}
