/* A C program used to send and receive a UDP datagram */
/* There is an optional verbose mode [-v] */
/* By Henry Fu */
 
#include <stdio.h> 
#include <stdlib.h> 
#include <errno.h> 
#include <string.h> 
#include <netdb.h> 
#include <sys/types.h> 
#include <netinet/in.h> 
#include <sys/socket.h> 

#define MYPORT 3490    /* the source port */
#define TOPORT 8128    /* the destination port */ 
#define MYSIZE 1536    /* the deault datagram size will be 1536 bytes */

/* Globals for getting command line arguments */
static char *options = "v";
static char *usage = "[input filename] [-v]";
int vflg = 0, errflg = 0;
char *host, *filename;
int port = TOPORT;
int myport = MYPORT;
int bufsize = MYSIZE;
int size;
unsigned char *buf, *rbuf, *lbuf;
int i;
int x0, x1, x2, x3, y0, y1, y2, y3;

/* Required by getopt */
extern char *optarg;
extern int optind, opterr;

/* Function to get command line arguments */
void parseargs(int argc, char *argv[])
{
    int c;
    int ok = 1;

    filename = argv[optind++];

    while ((c=getopt(argc,argv,options)) != -1)
        switch (c)
        {
        case 'v':
          vflg++;
          break;
        default:
          ok = 0;
        }
   
    if (!ok || optind < argc) {
      fprintf (stderr, "usage : %s %s\n", argv[0], usage);
      exit(0);
    }
}

/* Function to read input */
int parseinput()
{
    int i = 0;
    FILE *fd;

    if (filename != NULL) {
      if ((fd = fopen(filename, "r")) == NULL) {
        fprintf (stderr, "Cannot open file %s.\n", filename);
        exit(1);
      }
    }

    while (fgets(lbuf, 80, fd) != 0) { 
      switch (lbuf[0])
      {
      case '#':
        if (vflg) fprintf (stdout, "%s",lbuf);
        break;
      case '!':
        if (vflg) fprintf (stdout, "%s", lbuf);
        sscanf(lbuf, "!UDP %s %d %d\n", host, &port, &myport); 
        break;
      case '\n':
        if (vflg) fprintf (stdout, "%s", lbuf);
        break;
      default :
        if (vflg) fprintf (stdout, "%s", lbuf);
        sscanf(lbuf, "%2x%2x%2x%2x", &x0, &x1, &x2, &x3);
        buf[i] = (char) x0;
        buf[i+1] = (char) x1;
        buf[i+2] = (char) x2;
        buf[i+3] = (char) x3;
        i += 4;
        break;
      }
    }

    fclose(fd);

    if (vflg) fprintf( stdout, "size: %d\n", i);
    return i;
}

int main(int argc, char *argv[])
{
    int sockfd;  
    struct hostent *he;
    struct sockaddr_in my_addr;
    struct sockaddr_in their_addr; /* connector's address information */

    buf = calloc(bufsize, sizeof(char));
    rbuf = calloc(bufsize, sizeof(char));
    lbuf = calloc(80, sizeof(char));
    host = calloc(80, sizeof(char));

    parseargs(argc, argv);
    size = parseinput();

    if (vflg) fprintf (stdout, "Send is getting host name ...\n");

    if ((he=gethostbyname(host)) == NULL) {  /* get the host info */
        perror("Send failed at gethostbyname");
        exit(1);
    }

    if (vflg) fprintf (stdout, "Send is allocating socket ...\n");

    if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
        perror("Send failed at socket");
        exit(1);
    }

    my_addr.sin_family = AF_INET;
    my_addr.sin_port = htons(myport);
    my_addr.sin_addr.s_addr = INADDR_ANY;
    bzero(&(my_addr.sin_zero), 8);

    if (vflg) fprintf (stdout, "Send is binding ...\n");

    if (bind(sockfd, (struct sockaddr *)&my_addr, sizeof(struct sockaddr)) 
                                                                  == -1) {
        perror("Send failed at bind");
        exit(1);
    }

    their_addr.sin_family = AF_INET;      /* host byte order */
    their_addr.sin_port = htons(port);    /* short, network byte order */
    their_addr.sin_addr = *((struct in_addr *)he->h_addr);
    bzero(&(their_addr.sin_zero), 8);     /* zero the rest of the struct */

    if (vflg) fprintf (stdout, "Send is sending ...\n");

    if (sendto(sockfd, buf, size, 0, 
               (const struct sockaddr *) &their_addr, sizeof(their_addr)) < 0) {
        perror("Send failed at sendto");
        exit(1);
    }

    fprintf (stdout, "Sent: \n");
    for(i=0; i<size; i+=4) {
      x0 = buf[i];
      x1 = buf[i+1];
      x2 = buf[i+2];
      x3 = buf[i+3];
      fprintf (stdout, "%.2X%.2X%.2X%.2X\n",x0,x1,x2,x3);
    }

    close(sockfd);

    return 0;
}
